"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var path = require("path");
var execFile_endpoint_1 = require("../src/execFile-endpoint");
var first_last_endpoint_1 = require("../src/first-last-endpoint");
var casperjs_exec_1 = require("../src/casperjs-exec");
var casperEchoHelper = path.join('dist', 'test', 'helper', 'casper-echo.js');
describe('Endpoint casperJs', function () {
    it('can start and stop a casperJs process.', function (done) {
        var host_port = '127.0.0.1:45678';
        var eep = execFile_endpoint_1.default.command('casperjs', [casperEchoHelper, "--http_server=" + host_port]);
        var cjep = new casperjs_exec_1.default("http://" + host_port + "/", eep);
        var flep = new first_last_endpoint_1.default('_mi8o_', '_mi8o_', cjep);
        flep.input.subscribe(function (data) {
            if (data === '/startedServer') {
                flep.output.next('/shutdown');
            }
        }, function (err) {
            chai_1.assert.fail();
        }, function () {
            done();
        });
    });
    it('can start and stop multiple casperJs processes.', function (done) {
        var processPorts = ['45678', '45679', '45680', '45681', '45682'];
        function createCasperProcess(port, cb) {
            var host_port = "127.0.0.1:" + port;
            var eep = execFile_endpoint_1.default.command('casperjs', [casperEchoHelper, "--http_server=" + host_port]);
            var cjep = new casperjs_exec_1.default("http://" + host_port + "/", eep);
            var flep = new first_last_endpoint_1.default('_mi8o_', '_mi8o_', cjep);
            flep.input.subscribe(function (data) {
                if (data === '/startedServer') {
                    flep.output.next('/shutdown');
                }
            }, function (err) {
                chai_1.assert.fail();
            }, function () {
                cb(port);
            });
        }
        function assertDone(port) {
            var index = processPorts.indexOf(port);
            chai_1.assert.isTrue(index > -1);
            processPorts.splice(index, 1);
            if (processPorts.length === 0) {
                done();
            }
        }
        processPorts.forEach(function (port) { return createCasperProcess(port, assertDone); });
    });
    it('can recognise errors in casper process (webserver failed to start).', function (done) {
        var host_port = 'XXXXXXX';
        var eep = execFile_endpoint_1.default.command('casperjs', [casperEchoHelper, "--http_server=" + host_port]);
        var cjep = new casperjs_exec_1.default("http://" + host_port + "/", eep);
        var flep = new first_last_endpoint_1.default('_mi8o_', '_mi8o_', cjep);
        flep.input.subscribe(function (data) {
            chai_1.assert.fail('I will never be called');
        }, function (err) {
            chai_1.assert.deepEqual(err, { status: 1, signal: null, errors: ['/failedToStartServer'] });
            done();
        });
    });
    it('can recognise errors in casper process (error during execution).', function (done) {
        var host_port = '127.0.0.1:45678';
        var eep = execFile_endpoint_1.default.command('casperjs', [casperEchoHelper, "--http_server=" + host_port]);
        var cjep = new casperjs_exec_1.default("http://" + host_port + "/", eep);
        var flep = new first_last_endpoint_1.default('_mi8o_', '_mi8o_', cjep);
        flep.input.subscribe(function (data) {
            if (data === '/startedServer') {
                flep.output.next('/provokeEndpointError');
                return;
            }
            chai_1.assert.fail('never called');
        }, function (err) {
            chai_1.assert.deepEqual(err, { status: 1, signal: null, errors: ['/endpointError'] });
            done();
        });
    });
    it('can send messages back and forth between node and casperJs.', function (done) {
        var host_port = '127.0.0.1:45678';
        var eep = execFile_endpoint_1.default.command('casperjs', [casperEchoHelper, "--http_server=" + host_port]);
        var cjep = new casperjs_exec_1.default("http://" + host_port + "/", eep);
        var flep = new first_last_endpoint_1.default('_mi8o_', '_mi8o_', cjep);
        var count = 100;
        function loop(counter) {
            if (counter === 0) {
                flep.output.next('/shutdown');
                return;
            }
            flep.output.next("PING" + String(counter));
        }
        flep.input.subscribe(function (data) {
            if (data === '/startedServer') {
                loop(count);
            }
            else {
                chai_1.assert.equal(data, "PING" + String(count));
                loop(--count);
            }
        }, function (err) {
            chai_1.assert.fail();
        }, function () {
            done();
        });
    });
});
//# sourceMappingURL=casper-test.js.map